;---------------= Gray82 =--------------
; Author:       Ian Graf
;               (ian_graf@geocities.com)
; Port:         Sam Heald
;		    (void.calc.org)
; Version:      1.0
; Date:         4/22/00
;--------------= defines =--------------
; #define dbuffer
;  use double buffering
;  include buffer swapping routine
; 
; #define fastupdate
;  skip standard interrupt routine
;  ($0038) to create less flicker
;
; #define gsprite
;  include grayscale masked sprite
;  routine
;
; #define clip
;  clip grayscale masked sprite
;
; #define gcp
;  include grayscale picture
;  decompressing routine
;---------------------------------------


#define intcount        TEXT_MEM2           ; statvars
#ifdef  dbuffer
#define graybit1        intcount+1      ; grayscale screen 1
#define graybit2        graybit1+2      ; grayscale screen 1
#define graybuf1        graybit2+2      ; grayscale buffer 1
#define graybuf2        graybuf1+2      ; grayscale buffer 1

#else
#define graybit1        APD_BUF3           ; grayscale screen 1
#define graybit2        GRAPH_MEM         ; grayscale screen 1
#endif


#ifdef  gsprite
;------------= draw sprite =------------
; input:        ix - 8x8 masked
;                    grayscale sprite
;               bc - x,y location
; output:       sprite drawn

graysprite:
#ifdef  clip
        xor     a                       ;
        ld      de,8                    ;
        or      c                       ;
        jp      m,nclipb                ;
        ld      a,63                    ;
        sub     c                       ;
        ret     c                       ;
        cp      7                       ;
        jr      nc,nclipv               ;
        inc     a                       ;
        ld      e,a                     ;
        jr      nclipv                  ;
nclipb: cp      -7                      ;
        ret     c                       ;
        add     a,e                     ;
        ld      c,a                     ;
        sub     e                       ;
        neg                             ;
        ld      e,a                     ;
        add     ix,de                   ;
        ld      e,c                     ;
        ld      c,d                     ;
nclipv: ld      hl,$A07E                ; ld a,(hl) \ and b
        ld      (clipl),hl              ;
        inc     h                       ; ld a,(hl) \ and c
        ld      (clipr),hl              ;
        xor     a                       ;
        or      b                       ;
        jp      m,nclipr                ;
        cp      89                      ;
        jr      c,clipd                 ; 
        cp      96                      ;
        ret     nc                      ;
        ld      hl,$0218                ; jr 2
        ld      (clipr),hl              ;
        jr      clipd                   ;
nclipr: cp      -7                      ;
        ret     c                       ;
        ld      hl,$0218                ; jr 2
        ld      (clipl),hl              ;
clipd:  ld      hl,clipv                ;
        ld      (hl),e                  ;
        ld      b,d                     ;
        ld      h,b                     ;
        ld      l,c                     ;
        add     hl,bc                   ;
        add     hl,bc                   ;
        add     hl,hl                   ;
        add     hl,hl                   ;
        ld      c,a                     ;
        sra     c                       ;
        sra     c                       ;
        sra     c                       ;
        inc     c                       ;
        add     hl,bc                   ;
        dec     hl                      ;
        and     %00000111               ;
        ld      c,a                     ;
        push    ix                      ;
#else
        ld      d,0                     ;
        ld      e,c                     ;
        ld      h,d                     ;
        ld      l,c                     ;
        add     hl,de                   ;
        add     hl,de                   ;
        add     hl,hl                   ;
        add     hl,hl                   ;
        ld      a,b                     ;
        and     $07                     ;
        ld      c,a                     ;
        ld      e,b                     ;
        srl     e                       ;
        srl     e                       ;
        srl     e                       ;
        add     hl,de                   ;
#endif
        push    hl                      ;
        ld      a,16                    ;
        ld      (sprm),a                ;
#ifdef  dbuffer
        ld      de,(graybuf1)           ; write to inactive buffer 1
#else
        ld      de,graybit1             ; write to active buffer 1
#endif
        call    spr                     ;
        pop     hl                      ;
        ld      a,8                     ;
#ifdef  clip
        pop     ix                      ;
        ld      d,b                     ; b=0
        ld      e,a                     ; a=8
        add     ix,de                   ;
#endif
        ld      (sprm),a                ;
#ifdef  dbuffer
        ld      de,(graybuf2)           ; write to inactive buffer 2
#else
        ld      de,graybit2             ; write to active buffer 2
#endif
spr:    add     hl,de                   ;
clipv   =$+1
        ld      b,8                     ;
sl1:    push    bc                      ;
        ld      a,(ix+8)                ;
sprm    =$-1
        ld      b,a                     ;
        ld      d,(ix)                  ;
        inc     ix                      ;
        ld      a,c                     ;
        ld      c,$FF                   ;
        ld      e,0                     ;
        or      a                       ;
        jr      z,sl3                   ;
sl2:    srl     d                       ;
        rr      e                       ;
        scf                             ;
        rr      b                       ;
        rr      c                       ;
        dec     a                       ;
        jr      nz,sl2                  ;
clipl   =$
sl3:    ld      a,(hl)                  ;
        and     b                       ;
        or      d                       ;
        ld      (hl),a                  ;
        inc     hl                      ;
clipr   =$
        ld      a,(hl)                  ;
        and     c                       ;
        or      e                       ;
        ld      (hl),a                  ;
        ld      de,11                   ;
        add     hl,de                   ;
        pop     bc                      ;
        djnz    sl1                     ;
        ret                             ;
#endif


#ifdef  gcp
;----= decompress grayscale picture =---
; input:        ix - null terminated
;                    compressed
;                    grayscale picture
; output:       picture drawn

degcp:
#ifdef  dbuffer
        ld      hl,(graybuf1)           ;
        ld      de,(graybuf2)           ;
#else
        ld      hl,graybit1             ;
        ld      de,graybit2             ;
#endif
        ld      c,%10000000             ;
gcpl:   ld      a,(ix)                  ; get data
        inc     ix                      ; next byte
        or      a                       ; 0 -> finished
        ret     z                       ;
        push    de                      ;
        ld      d,a                     ; d -> data
        rlca                            ; 0 -> RLE
        jr      c,raw                   ; 1 -> RAW
rle:    ld      a,d                     ;
        and     %00011111               ;
        ld      b,a                     ; b -> count
        ld      a,d                     ;
rlel:   and     %01000000               ;
        jr      z,rlenp1                ;
        ld      a,(hl)                  ; plot graybit1
        or      c                       ;
        ld      (hl),a                  ;
rlenp1: ld      a,d                     ;
        pop     de                      ;
        push    af                      ;
        and     %00100000               ;
        jr      z,rlenp2                ;
        ld      a,(de)                  ; plot graybit2
        or      c                       ;
        ld      (de),a                  ;
rlenp2: rrc     c                       ;
        jr      nc,rlen                 ;
        inc     hl                      ;
        inc     de                      ;
rlen:   pop     af                      ;
        push    de                      ;
        ld      d,a                     ;
        djnz    rlel                    ;
        pop     de                      ;
        jr      gcpl                    ;
raw:    ld      b,3                     ; b -> count
rawl:   sla     d                       ;
        sla     d                       ;
        ld      a,d                     ;
        and     %10000000               ;
        jr      z,rawnp1                ;
        ld      a,(hl)                  ; plot graybit1
        or      c                       ;
        ld      (hl),a                  ;
rawnp1: ld      a,d                     ;
        pop     de                      ;
        push    af                      ;
        and     %01000000               ;
        jr      z,rawnp2                ;
        ld      a,(de)                  ; plot graybit2
        or      c                       ;
        ld      (de),a                  ;
rawnp2: rrc     c                       ;
        jr      nc,rawn                 ;
        inc     hl                      ;
        inc     de                      ;
rawn:   pop     af                      ;
        push    de                      ;
        ld      d,a                     ;
        djnz    rawl                    ;
        pop     de                      ;
        jr      gcpl                    ;
#endif


;-----------= clear buffers =-----------
; input:        none
; output:       buffers cleared

clrbuf:
#ifdef  dbuffer
        ld      hl,(graybuf1)           ; clear inactive buffer 1
        ld      d,h                     ;
        ld      e,l                     ;
        inc     de                      ;
#else
        ld      hl,graybit1             ; clear active buffer 1
        ld      de,graybit1+1           ;
#endif
        ld      (hl),0                  ;
        ld      bc,767                  ;
        ldir                            ;
#ifdef  dbuffer
        ld      hl,(graybuf2)           ; clear inactive buffer 1
        ld      d,h                     ;
        ld      e,l                     ;
        inc     de                      ;
#else
        ld      hl,graybit2             ; clear active buffer 1
        ld      de,graybit2+1           ;
#endif
        ld      (hl),0                  ;
        ld      bc,767                  ;
        ldir                            ;
        ret                             ;


#ifdef  dbuffer
;--------= swap active buffers =--------
; input:        none
; output:       active grayscale buffer
;               swaped with inactive
;               buffer

swpbuf: ld      hl,(graybuf1)           ; swap active buf with double buf
        ld      de,(graybit1)           ;
        ld      (graybuf1),de           ;
        ld      (graybit1),hl           ;
        ld      hl,(graybuf2)           ;
        ld      de,(graybit2)           ;
        ld      (graybuf2),de           ;
        ld      (graybit2),hl           ;
        ret                             ;
#endif


;---------= enable grayscale =----------
; input:        none
; output:       grayscale turned on.
;               buffers allocated and
;               cleared if double
;               buffered

grayscaleon:
#ifdef  dbuffer
        ld      hl,TEMP_MEM              ; points to last byte of used memory
        inc     hl                      ;
        ld      (graybuf1),hl           ; store buffer 1
        ld      de,768                  ;
        add     hl,de                   ;
        ld      (graybuf2),hl           ; store buffer 2
        ld      hl,APD_BUF3               ;
        ld      (graybit1),hl           ; store graybit 1
        ld      hl,GRAPH_MEM              ;
        ld      (graybit2),hl           ; store graybit 2
        call    clrbuf                  ; clear buffers
        call    swpbuf                  ; swap to active
#endif
        ld      hl,$8400                ; set up interrupt vector
        ld      de,$8401                ;
        ld      (hl),$82                ;
        ld      bc,$0100                ;
        ldir                            ;
        ld      hl,$8282                ;
        ld      (hl),$C3                ; jp greyscale
        inc     hl                      ;
        ld      (hl),grayint&$00FF      ;
        inc     hl                      ;
        ld      (hl),grayint>>8         ;
        xor     a                       ; reset counter
        ld      a,(intcount)            ;
        ld      a,$84                   ; point i to interrupt vector
        ld      i,a                     ;
        im      2                       ;
        ret                             ;


;---------= disable grayscale =---------
; input:        none
; output:       grayscale turned off

grayscaleoff:
        im      1                       ;
        ret                             ;


;--------= grayscale interrupt =--------

grayint:
        ex      af,af'                  ;
        exx                             ;
        in      a,(3)                   ; check vbl int
        and     %00000010               ;
        jr      z,exit_interrupt        ;
        ld      hl,intcount             ; inc counter
        inc     (hl)                    ;
        ld      a,(hl)                  ;
        dec     a                       ; 1
        jr      z,dpic2                 ;
        dec     a                       ; 2
        jr      z,dpic1                 ;                     ;
        ld      (hl),0                  ; reset counter
exit_interrupt:
        in      a,(3)                   ; check on interrupt status
        rra                             ;
        ld      a,0                     ;
        adc     a,9                     ;
        out     (3),a                   ;
        ld      a,$0B                   ;
        out     (3),a                   ;
        exx                             ;
        ex      af,af'                  ;
#ifdef  fastupdate
        ei                              ; skip standard interrupt
        reti                            ;
#else
        jp      $0038                   ; call standard interrupt handler
#endif

dpic1:
#ifdef  dbuffer
        ld      hl,(graybit1)           ; display graybit 1
#else
        ld      hl,graybit1             ;
#endif
        jr      drawp                   ;

dpic2:
#ifdef  dbuffer
        ld      hl,(graybit2)           ; display graybit 2
#else
        ld      hl,graybit2             ;
#endif

drawp:  ld      a,$80                   ;
        out     ($10),a                 ;
        ld      de,755                  ;
        add     hl,de                   ;
        ld      a,$20                   ;
        ld      c,a                     ;
wloop1: ld      b,64                    ;
        inc     c                       ;
        ld      de,-767                 ;
        out     ($10),a                 ;
        add     hl,de                   ;
        ld      de,11                   ;
wloop2: add     hl,de                   ;
        inc     hl                      ;                      ;
        ld      a,(hl)                  ;
        out     ($11),a                 ;
	  push ix
	  pop ix
        djnz    wloop2                  ;
        ld      a,c                     ;
        cp      $2B+1                   ;
        jr      nz,wloop1               ;
        jr      exit_interrupt 
         ;
APD_BUF3:
 .ds 768
#ifdef  dbuffer
TEMP_MEM:
 .ds 1536
#endif
 .db 0